@@ -1287,6 +1287,8 @@ public:
 	Gpos= new MatrixXd(prob.l,prob.l);
 	MatrixXd G(prob.l,prob.l); 
 	
+	l = prob.l;
+	
 	for (int i = 0; i < prob.l; i++)
 	  for (int j = 0; j < prob.l; j++)
 	    G(i,j) = (Qfloat)(y[i]*y[j]*(this->*kernel_function)(i,j));
@@ -1302,12 +1304,21 @@ public:
 	for (int i = 0; i < prob.l; i++) 
 	  D(i,i) = eigenvals(i, 0);
 	
+	bool checkPos = false;
+	
 	// eigenvalues all positives
 	for (int i = 0; i < prob.l; i++)
-	  if (D(i,i) < 0)
+	  if (D(i,i) < -1.0e-6) {
+	    checkPos = true;
 	    D(i,i) = -D(i,i);
+	  }
 	  
-	  MatrixXd result = eigenvecs * D * eigenvecs.inverse();
+	  MatrixXd result;
+	  
+	  if(checkPos) 
+	    result = eigenvecs * D * eigenvecs.inverse();
+	  else
+	    result = G;
 	  
 	  // it copies the content of matrix result into Gpos
 	  Gpos->operator=(result);
@@ -1318,6 +1329,8 @@ public:
 	Qfloat *get_Q(int i, int len) const
 	{
 		Qfloat *data;
+	  
+//	  l;
 		int start;
 		if((start = cache->get_data(i,&data,len)) < len)
 		{
@@ -1353,6 +1366,7 @@ private:
 	Cache *cache;
 	double *QD;
 	MatrixXd *Gpos;
+	int l;
 };
 
 class ONE_CLASS_Q: public Kernel
@@ -3228,84 +3242,3 @@ void svm_set_print_string_function(void (*print_func)(const char *))
 		svm_print_string = print_func;
 }
 
-
-
-class PRE_SVC_Q: public Kernel
-{ 
-public:
-  PRE_SVC_Q(const svm_problem& prob, const svm_parameter& param, const schar *y_)
-    :Kernel(prob.l, prob.x, param)
-  {
-    clone(y,y_,prob.l);
-    cache = new Cache(prob.l,(long int)(param.cache_size*(1<<20)));
-    QD = new double[prob.l];
-    for(int i=0;i<prob.l;i++)
-      QD[i]= (double)(this->*kernel_function)(i,i);
-    
-    Gpos= new MatrixXd(prob.l,prob.l);
-    MatrixXd G(prob.l,prob.l); 
-    
-    for (int i = 0; i < prob.l; i++)
-      for (int j = 0; j < prob.l; j++)
-        G(i,j) = (Qfloat)(y[i]*y[j]*(this->*kernel_function)(i,j));
-    
-    SelfAdjointEigenSolver<MatrixXd> solver(G);
-    
-    MatrixXd eigenvals = solver.eigenvalues();  // vector of eigenvalues
-    MatrixXd eigenvecs = solver.eigenvectors(); // column matrix of eigenvectors
-    
-    MatrixXd D = MatrixXd::Zero(prob.l,prob.l);
-    
-    // creating diagonal matrix of eigenvalues
-    for (int i = 0; i < 3; i++) 
-      D(i,i) = eigenvals(i, 0);
-    
-    // eigenvalues all positives
-    for (int i = 0; i < 3; i++)
-      if (D(i,i) < 0)
-        D(i,i) = -D(i,i);
-      
-      MatrixXd result = eigenvecs * D * eigenvecs.inverse();
-      
-      // it copies the content of matrix result into Gpos
-      Gpos->operator=(result);
-  }
-  
-  Qfloat *get_Q(int i, int len) const
-  {
-    Qfloat *data;
-    int start;
-    if((start = cache->get_data(i,&data,len)) < len)
-    {
-      for(int j=start;j<len;j++)
-        data[j] = Gpos->operator()(i,j); // modified
-    }
-    return data;
-  }
-  
-  double *get_QD() const
-  {
-    return QD;
-  }
-  
-  void swap_index(int i, int j) const
-  {
-    cache->swap_index(i,j);
-    Kernel::swap_index(i,j);
-    swap(y[i],y[j]);
-    swap(QD[i],QD[j]);
-  }
-  
-  ~PRE_SVC_Q()
-  {
-    delete[] y;
-    delete cache;
-    delete[] QD;
-    delete Gpos;
-  }
-private:
-  schar *y;
-  Cache *cache;
-  double *QD;
-  MatrixXd *Gpos;
-};
